import urllib
import ImageFile

from django.core.cache import cache

from celery.decorators import task


def get_image_size(uri):
    file = urllib.urlopen(uri)
    p = ImageFile.Parser()
    data = file.read()#file.read(1024)
    if not data:
        return None
    p.feed(data)
    if p.image:
        return p.image.size
    file.close()
    #not an image
    return None

@task
def fetch_dimensions(url):
    dim = get_image_size(url)
    key = urllib.quote('imagedimensions-%s'%url)
    if dim:
        cache.set(key, dim)
    else:
        cache.set(key, (0,0))



from imagedimensions import add_dimensions

class ImageDimensionsMiddleware(object):

    def process_response(self, request, response):        
        if 'text/html' in response['Content-Type']:
            path = request.build_absolute_uri()
            response.content, dimensions_found = add_dimensions(response.content, path)
        return response



import re

import urllib, urlparse

from bs4 import BeautifulSoup

from django.core.cache import cache

from imagedimensions.tasks import fetch_dimensions

RE_IMG = re.compile('<img.*?/>')


def add_dimensions(html, base_uri):    
    class replace_img:
        def __init__(self):
            self.dimensions_found = True
        def __call__(self, match):
            img_str = match.group(0)
            img = BeautifulSoup(img_str).img

            if 'width' in img.attrs or 'height' in img.attrs:
                return img_str
            url = urlparse.urljoin(base_uri, img['src'])
            dim  = cache.get('imagedimensions-%s'%urllib.quote(url))
            if dim is None:
               fetch_dimensions.delay(url)
               self.dimensions_found = False           
               return img_str
            w, h = dim
            if w and h:
                img['width'] = w
                img['height'] = h
            return str(img)
    replacer = replace_img()
    val = re.sub(RE_IMG, replacer, html)
    return val, replacer.dimensions_found
